@@ -1,10 +1,13 @@
 use serde::Deserialize;
 use std::{iter::once, mem};
+use swc_atoms::js_word;
 use swc_common::{util::take::Take, Spanned, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::perf::Check;
 use swc_ecma_transforms_macros::fast_path;
-use swc_ecma_utils::{alias_if_required, prepend, private_ident, undefined, ExprFactory, StmtLike};
+use swc_ecma_utils::{
+    alias_if_required, prepend, private_ident, undefined, ExprFactory, IntoIndirectCall, StmtLike,
+};
 use swc_ecma_visit::{
     as_folder, noop_visit_mut_type, noop_visit_type, Fold, Visit, VisitMut, VisitMutWith,
 };
@@ -509,8 +512,16 @@ impl OptChaining {
                     _ => false,
                 };
 
-                let (left, right, alt) = match **obj {
-                    Expr::Ident(..) => (obj.clone(), obj.clone(), e.expr.take()),
+                let (left, right, alt) = match &**obj {
+                    Expr::Ident(ident) => (
+                        obj.clone(),
+                        obj.clone(),
+                        if ident.sym == js_word!("eval") {
+                            Box::new(e.expr.take().expect_call().into_indirect().into())
+                        } else {
+                            e.expr.take()
+                        },
+                    ),
                     _ if is_simple_expr(&obj) && self.c.pure_getter => {
                         (obj.clone(), obj.clone(), e.expr.take())
                     }
